#!/bin/bash
# **************************************************************** #
# Licensed Materials - Property of IBM                             #
# 5725D70                                                          #
# (C) Copyright IBM Corp.  2012, 2013    All Rights Reserved.      #
# US Government Users Restricted Rights - Use, duplication or      #
# disclosure restricted by GSA ADP Schedule Contract with          #
# IBM Corp.                                                        #
# **************************************************************** #

usage(){
	echo -e "Usage:"
	echo -e "\t -h print this usage"
	echo -e "\t -a <datadir> load all infrastructure data, equals -i -v"
	echo -e "\t -c <configuration> default to offline.properties"
	echo -e "\t -i <datadir> load infrastructure data, directory should contain route.csv, route shapes, route stops and journey patterns"
	echo -e "\t -e <datadir> load ESRI format shape file, directory should contain at least route.shp,route.prj and route.dbf, only valid with -i"
	echo -e "\t -v <datadir> load vehicle data, directory should contain vehicle.csv, vehicletype.csv"
	echo -e "\t -p <datadir> load production timetable, directory should contain files named like <yyyy-MM-dd><*>productiontimetable.xml, e.g. 2012-11-11_productiontimetable.xml"
	echo -e "\t -f force mode, no DB check will be performed, this should only be used when error occurs in the stage 2 and after errors are fixed"
	echo -e "\t -l <confidencelevel.xml> load confidence level"
	echo -e "\t -t <seconds> specify forecast expire time, default to 300 seconds"
	echo -e "\t -n <StopLinkID> StopLinkID is the column name defined in stoplink_shape.dbf,which contains ID of stop link shape, only valid with -e"
	echo -e "\t -d <dir> specify temporary directory to save converted stop link shape file, only valid with -e"
	echo -e "\t -w <password> database password, user will be prompted to input database password if -w is not provided "
	exit 0
}

if [ $# -eq 0 ]
then
	usage
fi

DB_NAME="TIH_HS"
CFG_FILE="offline.properties"

while [ $# -ne 0 ]
do
    # if OPTIND > $#+1, getopts will not change OPTIND's value,so set it to 0
    OPTIND=0
    while getopts h:a:c:i:e:f:v:p:l:t:n:d:w:z:r opt
    do
        case $opt in
	    h)
	    usage
	    ;;	
	    a)
	    INFRASTRUCTURE=$OPTARG
	    VEHICLE=$OPTARG	   
	    CONFIDENCE_LEVEL=confidencelevel.xml
	    ;;
	    c)
	    CFG_FILE=$OPTARG
	    ;;
	    f)
	    FORCE_MODE='Y'
	    ;;
	    i)
	    INFRASTRUCTURE=$OPTARG
	    ;;
	    e)
        ESRI_DIR=$OPTARG
	    ;;	    
	    v)
	    VEHICLE=$OPTARG
	    ;;
	    p)
	    PRODUCTION_TIMETABLE=$OPTARG
	    ;;
	    l)
	    CONFIDENCE_LEVEL=$OPTARG
	    ;;
	    t)
	    FORECAST_EXPIRE=$OPTARG
	    ;;
	    d)
	    TEMP_DIR=$OPTARG
	    ;;
	    n)
	    ROUTEID=$OPTARG
	    ;;
	    r)
	    CLEARDB='Y'
	    ;;
	    z)
	    DB_NAME=$OPTARG
            ;;
	    w)
	    PASSWD=$OPTARG
	    ;;
            \?)
	    usage
            ;;
        esac
    done
 
    # if getopts find a non-option argument
    # it will stop parsing and return OPTIND
    # as index to the first non-option argument
    # 
    # if getopts doesn't find any non-option argument
    # it will set OPTIND=$#+1
    if [ $OPTIND -ne $(($#+1)) ]
    then
	usage
    else
        break; # getopts doesn't find any non-option argument
    fi
done


echo "CONFIG_FILE="$CFG_FILE
if [ ! -f ${CFG_FILE} ]
then
	echo "${CFG_FILE} does not exist"
	exit 1
fi

ARGUMENTS="${ARGUMENTS} -config ${CFG_FILE}" 

if [ -z ${PASSWD}  ]
then
	read -s -p "Enter Database Password:" dbpasswd
	echo -e "\n"
else
	dbpasswd=${PASSWD}
fi

ARGUMENTS="${ARGUMENTS} -password ${dbpasswd}"

if [ $INFRASTRUCTURE ]
then	
		if [ -f "${INFRASTRUCTURE}/boundingbox.csv" ] || [ -f "${INFRASTRUCTURE}/direction.csv" ] || [ -f "${INFRASTRUCTURE}/employee.csv" ] || [ -f "${INFRASTRUCTURE}/groupofoperator.csv" ] || [ -f "${INFRASTRUCTURE}/journeypattern.csv" ] || [ -f "${INFRASTRUCTURE}/line.csv" ] || [ -f "${INFRASTRUCTURE}/ operator.csv" ] || [ -f "${INFRASTRUCTURE}/route.csv" ] || [ -f "${INFRASTRUCTURE}/service.csv" ] || [ -f "${INFRASTRUCTURE}/specifiedstoplink.csv" ] || [ -f "${INFRASTRUCTURE}/stop.csv" ] || [ -f "${INFRASTRUCTURE}/stoplink_idx.csv" ] || [ -f  "${INFRASTRUCTURE}/stoplink_shape.csv" ] || [ -f "${INFRASTRUCTURE}/vehicle.csv" ] || [ -f "${INFRASTRUCTURE}/vehicletype.csv" ]
        then
                echo "INFRASTRUCTURE_DATA_DIR= ${INFRASTRUCTURE}"
                ARGUMENTS="${ARGUMENTS} -infrastructure ${INFRASTRUCTURE}"
        else
                echo "${INFRASTRUCTURE} is not a valid infrastructure data directory"
                exit 1
        fi
fi
 

if [ ${VEHICLE} ]
then
	if [ -f "${VEHICLE}/vehicle.csv" ] || [ -f "${VEHICLE}/vehicletype.csv" ]
        then
                echo "VECHILE_DATA_DIR= ${VEHICLE}"
                ARGUMENTS="${ARGUMENTS} -vehicle ${VEHICLE}"
        else
                echo "${VEHICLE} is not a valid vehicle directory"
                exit 1
        fi
fi


if [ ${PRODUCTION_TIMETABLE} ]
then
	files=$(ls ${PRODUCTION_TIMETABLE}/*productiontimetable.xml 2> /dev/null | wc -l)
	if [ "$files" != "0" ]
        then
                echo "PRODUCTION_TIMETABLE= ${PRODUCTION_TIMETABLE}"
                ARGUMENTS="${ARGUMENTS} -productiontimetable ${PRODUCTION_TIMETABLE}"
        else
                echo "no production time table files are found in ${PRODUCTION_TIMETABLE}"
                exit 1
        fi
fi

if [ ${CONFIDENCE_LEVEL} ]
then
	if [ -f ${CONFIDENCE_LEVEL} ]
        then
                echo "CONFIDENCE_LEVEL= ${CONFIDENCE_LEVEL}"
                ARGUMENTS="${ARGUMENTS} -confidencelevel ${CONFIDENCE_LEVEL}"
        else
                echo "${CONFIDENCE_LEVEL} does not exist"
                exit 1
        fi
fi

if [ ${CLEARDB} ]
then
	ARGUMENTS="${ARGUMENTS} -cleardb Y"
fi

if [ ${FORCE_MODE} ]
then
	ARGUMENTS="${ARGUMENTS} -force Y"
fi

if [ ${FORECAST_EXPIRE} ]
then
	ARGUMENTS="${ARGUMENTS} -forecastexpire ${FORECAST_EXPIRE}"
fi

if [ ${ESRI_DIR} ] && [ ${INFRASTRUCTURE}  ]
then
        if [ -f "${ESRI_DIR}/stoplink.shp" ]
        then
		if [ -z ${ROUTEID}  ] 
		then 
			echo "-n must be provided if -e is enabled"
			exit 1
		fi
		ARGUMENTS="${ARGUMENTS} -stoplinkid ${ROUTEID}"
		CUR_DIR=`pwd`
		cd ${ESRI_DIR}
		ESRI_DIR=`pwd`
		cd ${CUR_DIR}
		echo "ESRI_SHAPE_DATA_DIR= ${ESRI_DIR}"
		echo "start converting esri shape file..."
		db2 CONNECT TO ${DB_NAME}
		db2 DROP TABLE GEOSPATIAL.TMPGEOTABLE > /dev/null
        sh ./importShape.sh -f ${ESRI_DIR}/stoplink -t TMPGEOTABLE -c GEOCOLUMN -d ${DB_NAME}
        if [ $? -ne 0 ]
        then
                echo "Convert esri shape failed"
                exit 1
		else
				echo "Convert esri shape successful"
	                fi

                ARGUMENTS="${ARGUMENTS} -shapefromdb TMPGEOTABLE"
        else
                echo "${ESRI_DIR} is not a valid route shape directory"
                exit 1
        fi
fi

if [ ${TEMP_DIR} ] && [ ${INFRASTRUCTURE} ]
then
	[[ -d ${TEMP_DIR} ]] || mkdir -p ${TEMP_DIR}
        ARGUMENTS="${ARGUMENTS} -tmpdir ${TEMP_DIR}"
fi
#echo "ARGS:${ARGUMENTS}"


if [ -z "$DB2_HOME" ]
then
        DB2_HOME=/opt/IBM/DB2
fi

if [ -z "$JAVA_HOME" ]
then
        JAVA_HOME=$DB2_HOME/java/jdk64/jre
fi

if [ -z "$TRANS_COMMON_LIB" ]
then
        TRANS_COMMON_LIB=/opt/IBM/iss/intelligentTransportation/intelligentTransitAnalytics/scripts
fi

MAIN=com.ibm.iss.trans.vap.infrastructure.Loader

$JAVA_HOME/bin/java -Xms512m -Xmx4096m -verbose:gc -verbose:debug -verbose:stack -classpath $CLASSPATH:${DB2_HOME}/java/db2jcc4.jar:$TRANS_COMMON_LIB/trans_vap_offline_base.jar:$TRANS_COMMON_LIB/trans_vap_common_resource.jar:$TRANS_COMMON_LIB/trans_vap_offline_infrastructure.jar $MAIN ${ARGUMENTS}

if [ ${ESRI_DIR} ]
then
	echo "Deleting temporary table"
	db2 DELETE FROM GEOSPATIAL.TMPGEOTABLE
	db2 DROP TABLE GEOSPATIAL.TMPGEOTABLE
fi
